/*----------------------------------------------------------------------
 | VAR.H							940420
 |
 | Contains all global procedure prototypes and variable prototypes.
 +----------------------------------------------------------------------*/



#include "macro.h"

#ifdef _WIN32

#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE 
#define _SCL_SECURE_NO_DEPRECATE 
#define _CRT_SECURE_NO_DEPRECATE


#else
#define CDECL
#endif

#define bool int
#define true 1
#define false 0


static void CDECL catch_sigint(int s);

static void     CDECL
                catch_sigint(int s)
{
    signal(s, catch_sigint);
}



#define     VERSION         "1.16wb JA"
#define     BUILDDATE       "14-01-09"

#define INI_FILENAME "alibaba.ini"


/* piece values */


static int pawn_value    =   0;
static int knight_value  =   0;    
static int bishop_value  =   0;         
static int rook_value    =   0;
static int queen_value   =   0;
static int king_value    =   0; 


static bool using_bookthinker  = false;
static bool using_resign       = false;
static bool using_draw_offer   = false;
   

static int use_bookthinker  = 0;
static int bookthinker_use  = 0;
static int use_resign       = 0;
static int resign_use       = 1;
static int use_draw_offer   = 0;
static int draw_offer_use   = 1;
static int use_hashsize     = 0;
static int hashsize_use     = 0;
static int value_pawn       = 100;
static int value_knight     = 325;
static int value_bishop     = 325;
static int value_rook       = 520;
static int value_queen      = 900;
static int value_king       = 3200;


static bool  xboard_mode       = false;
static bool  print_board       = false;
static bool  first_move        = false;
static bool  ftime_ok          = false;
static bool  st_time           = false;
static bool  time_set          = false;
static bool  force_mode        = false;
static bool  draw_offer        = false;
static bool  endgame_values    = false;
static bool  Stop_Search       = false;
static bool  post              = true;
static bool  memory_on         = true;
static bool  icc               = false;
static int   maxdepth          = 32;
static int   max_moves         = 40;
static int   minutes           = 0;
static int   move_count        = 0;
static int   resign_count      = 0;
static int   mymaxdepth        = 0;
static int   memory            = 0;
static int   hashsize          = 0;

static char  wb_command[256];
static char  line[256];

static int start_time         = 0;
static int maxtime            = 0;
static int inc_time           = 0;
static int stop_time          = 0;
static int time_left_opponent = 0;

static ValueType       value;
static char symbolType[100];

extern Unsigned1 nrTtBits;



/* hash scheme keep method - 1=on 0=off (choose only one) */

static int Use_New       = 0;
static int Use_Big1      = 0;
static int Use_BigAll    = 0;
static int Use_Deep      = 0;
static int Use_Old       = 0;


static int new_use    = 1;
static int big1_use   = 0;
static int bigall_use = 0;
static int deep_use   = 0;
static int old_use    = 0;

/*
old:
    in case of a collision, the oldest position is kept in the table. Thus,
    a position is *never* overwritten.
  new:
    in case of a collision, the newest position is kept in the table. Thus,
    a position is *always* overwritten.
  deep:
    in case of a collision, the position with the deepest subtree is kept
    in the table
  big1:
    in case of a collision, the position with the biggest subtree (in terms
    of number of nodes) is kept in the table. A position read from the
    transposition table is counted as one node.
  bigAll:
    in case of a collision, the position with the biggest subtree (in terms
    of number of nodes) is kept in the table. A position read from the
    transposition table is counted as the number of nodes it had been searched
    for.
  . EXAMPLE: ab -scheme new
      always replaces a position in case of a collision.
  . DEFAULT: big1

*/




// ***** start of alibaba.ini *****

#define NUM_KEYS 200


char iniKeys[NUM_KEYS][150] = {
"use_bookthinker","use_resign","use_draw_offer", "use_hashsize",
"pawn_value","knight_value","bishop_value","rook_value","queen_value",
"king_value","Use_New","Use_Big1","Use_BigAll","Use_Deep","Use_Old"


};

void ReadINIFile(char *filename)
{
FILE *in;
bool foundKey[NUM_KEYS];
char buffer[4000];
char key[120];
int c,i;

use_bookthinker = use_resign = use_draw_offer = use_hashsize =
pawn_value = knight_value = bishop_value = rook_value = queen_value =
king_value = Use_New = Use_Big1 = Use_BigAll = Use_Deep = Use_Old = 0;


memset(foundKey,false, sizeof(bool)*NUM_KEYS);
memset(buffer,0,sizeof(char) * 4000);

in = fopen(filename, "r");
if(in == NULL) {
printf("\nNo %s file found - setting internal default values.\n\n", filename);
for(i = 0; i < NUM_KEYS; i++) {
}
return;
} else {
printf("\n%s found. Loading values....   \n\n", filename);	
}

for(;;) {
c = fgetc(in);
if(c == '*') {
fscanf(in, "%s %s", key, buffer);
if((i = KeyNum(key)) >= 0) { // test that key is legit
SetINIKey(key, buffer, false);
foundKey[i] = true;
memset(buffer,0,sizeof(char) * 4000);
}
}
if(c == EOF)  {
printf("\ndone. \n\n");	
break;
}
}

//	 Find all keys whose values were no explicitely set in the ini file and set them to the default
for(i = 0; i < NUM_KEYS; i++) {
if(foundKey[i] == false) SetINIKey(iniKeys[i], NULL, true);
}
fclose(in);	
}

int KeyNum(char *key) {
int i;
for(i = 0; i < NUM_KEYS; i++) {
if(!strcmp(key, (char *)iniKeys[i])) return i;
}
return -1;
}

//***** set values from alibaba.ini ******
int SetINIKey(char *key, char *buffer, bool default_val)
{

if (!strcmp(key, "use_bookthinker")) {
		bookthinker_use = atoi(buffer);
        if (bookthinker_use) {
        using_bookthinker = true;                    
        printf("\n[alibaba] using bookthinker selected\n");
        }}
				 										         	    									
if (!strcmp(key, "use_resign")) {
		resign_use = atoi(buffer);
        if (resign_use) {
        using_resign = true;                    
        printf("\n[alibaba] using resign selected\n");
        }}
        
if (!strcmp(key, "use_draw_offer")) {
		draw_offer_use = atoi(buffer);
        if (draw_offer_use) {
        using_draw_offer = true;                    
        printf("\n[alibaba] using offer-drw selected\n");
        }}       
        
        
if (!strcmp(key, "use_hashsize")) {
		hashsize_use = atoi(buffer);
        if (hashsize_use) {                  
        printf("\n[alibaba] using hashsize %dmb selected\n", hashsize_use);
        if (hashsize_use == 3)  {
        hashsize = 16; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 6)  {
        hashsize = 17; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 12) { 
        hashsize = 18; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 22) {  
        hashsize = 20; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 32) {  
        hashsize = 19;
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 82) { 
        hashsize = 22; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else
        if (hashsize_use == 164) {
        hashsize = 23; 
        nrTtBits = hashsize;   
        memory_on = false;
        }else{
        printf("[alibaba] hash size selected not supported\n");
        printf("[alibaba] using default 32mb hash size\n");
        }
        }}  
        
        if (!strcmp(key, "pawn_value")) {
		value_pawn = atoi(buffer);
        if (value_pawn) {                    
        printf("\n[alibaba] pawn value   = %d\n",value_pawn);
        }}       
                     
        if (!strcmp(key, "knight_value")) {
		value_knight = atoi(buffer);
        if (value_knight) {                    
        printf("\n[alibaba] knight value = %d\n",value_knight);
        }}        
        
        if (!strcmp(key, "bishop_value")) {
		value_bishop = atoi(buffer);
        if (value_bishop) {                    
        printf("\n[alibaba] bishop value = %d\n",value_bishop);
        }}        
        
        if (!strcmp(key, "rook_value")) {
		value_rook = atoi(buffer);
        if (value_rook) {                    
        printf("\n[alibaba] rook value   = %d\n",value_rook);
        }}        
        
        if (!strcmp(key, "queen_value")) {
		value_queen = atoi(buffer);
        if (value_queen) {                    
        printf("\n[alibaba] queen value  = %d\n",value_queen);
        }}        
        
        if (!strcmp(key, "king_value")) {
		value_king = atoi(buffer);
        if (value_king) {                    
        printf("\n[alibaba] king value   = %d\n",value_king);
        }}        
        
        if (!strcmp(key, "Use_New")) {
		new_use = atoi(buffer);
        if (new_use) {                    
        printf("\n[alibaba] hashtable keep method = new\n");
        }}        

        if (!strcmp(key, "Use_Big1")) {
		big1_use = atoi(buffer);
        if (big1_use) {                    
        printf("\n[alibaba] hashtable keep method = big1\n");
        }}        

        if (!strcmp(key, "Use_BigAll")) {
		bigall_use = atoi(buffer);
        if (bigall_use) {                    
        printf("\n[alibaba] hashtable keep method = bigall\n");
        }}        

        if (!strcmp(key, "Use_Deep")) {
		deep_use = atoi(buffer);
        if (deep_use) {                    
        printf("\n[alibaba] hashtable keep method = deep\n");
        }}        

        if (!strcmp(key, "Use_Old")) {
		old_use = atoi(buffer);
        if (old_use) {                    
        printf("\n[alibaba] hashtable keep method = old\n");
        }}        





return 0;
}

//***** end of alibaba.ini  ****************










/*----------------------------------------------------------------------
 | INIT							940420
 |
 | Contains procedures concerning the initialization of several
 | variables.
 +----------------------------------------------------------------------*/



static void     InitMoveTables(void);
static void     InitKnightTable(void);
static void     InitBishopTable(void);
static void     InitRookTable(void);
static void     InitPawnMoveTables(void);
static void     InitPawnCaptureTables(void);
static void     InitPieceSymbols(void);
static void     InitPieceValues(void);
static void     InitAllocVariables(void);
static void     FillHashTables(void);
static void     InitPackArrays(void);
static void     UpdateCentreTable(void);
static Boolean  Unique(HashType h);
static void     CheckHashTables(void);



static int             get_ms(void)
{
    struct timeb    timebuffer;
    ftime(&timebuffer);
    if (timebuffer.millitm != 0)
        ftime_ok = true;
    return (timebuffer.time * 1000) + timebuffer.millitm;
    }


/* ab.c */
extern void     Search(Signed1 depth, MoveType * move, ValueType * value);
extern void     FreePath(PathType * path);

/* command.c */
extern Signed2  InterpretCommand(void);
extern void     PerformMoveCommand(char *moveString);
extern void     PerformPlacePiece(char *pieceString);

/* diskio.c */
extern void     SaveGame(char *filename);
extern void     LoadGame(char *filename);
extern void     SaveLatexPos(char *filename);

/* evaluate.c */
extern ValueType Evaluate(Signed1 ttDepth, ValueType ttValue, Unsigned1 flag,
                                          Unsigned4 ttNodes);

/* generate.c */
extern void     GenerateMoves(MoveType ** moveList, Unsigned1 * nrMoves);
extern Boolean  Attacked(BoardIndexType square, SquareType color);
#ifdef Bcc
extern Boolean  FindFirstPieceInDir(Unsigned1 max,
                   BoardIndexType huge * table, PieceType * attackingPiece);
#else
extern Boolean  FindFirstPieceInDir(Unsigned1 max, BoardIndexType * table,
                                                PieceType * attackingPiece);
#endif
extern void     SearchForPinsAndCheck(void);
extern void     GenerateCaptureMoves(CheckPieceType * piece, MoveType * list,
                                                     Unsigned1 * nrMoves);
extern void     GenerateInterposingMoves(CheckPieceType * piece, MoveType * list,
                                                       Unsigned1 * nrMoves);
extern void     GenerateKingMoves(BoardIndexType square, MoveType * list,
                                                  Unsigned1 * nrMoves);
extern Signed2  CompareMoves(MoveType * move1, MoveType * move2);
extern void     HeapSort(MoveType * moveList, Unsigned1 nrMoves);
extern void     SiftDown(MoveType * moveList, Unsigned1 root, Unsigned1 leaf);

/* init.c */
extern void     Init(void);
extern void     EmptyBoard(void);
extern void     StartingPosition(void);
extern void     PlacePiece(SquareType piece, BoardIndexType square);
extern void     ClearHistoryTables(void);
extern void     ClearPieceSquareTables(void);
extern void     InitPositionalValue(void);
extern void     InitTransTable(void);

/* io.c */
extern void     PrintBoard(void);
extern void     PrintMoveList(MoveType * list, Unsigned1 nrMoves);
extern void     PrintAllocVariables(void);
extern void     PrintGame(Signed2 from, Signed2 to);
extern void     PrintPath(PathType * path);
extern void     PrintHistoryTable(void);
extern void     PrintHashes(void);
extern void     PrintSettings(void);
extern void     PrintTestSettings(Signed2 testDepth, Signed2 testColor,
                                                  Signed4 testNrNodes);

/* move.c */
extern void     DoMove(MoveType * move);
extern void     UndoMove(void);

/* quies.c */
extern ValueType Quiescence(ValueType alpha, ValueType beta,
                   PathType ** bestPath, Signed1 ttDepth, ValueType ttValue,
                                         Unsigned1 flag, Unsigned4 ttNodes);

/* search.c */
extern void     PreProcessing(void);
extern Signed2  ItIsADraw(void);

/* testing.c */
extern void     TestArguments(void);


extern BoardIndexType lastMovedPieceSquare;
extern Unsigned1 nrCheck;
extern CheckPieceType checkingPiece[2];
extern DirectionType pinned[H8 + 1];
extern SquareType board[NrSquares];
extern BoardIndexType epSquare[MaxGamePlies];
extern Unsigned1 castling[MaxGamePlies];
extern HashType hashValue[MaxGamePlies];
extern Unsigned2 startHashIndex[MaxGamePlies];
extern SquareType toMove;
extern Unsigned4 material[2];
extern ValueType positionalValue[2];
extern Unsigned2 pieceValue[128];
extern char     pieceSymbol[256];
extern PieceType pieces[2][16];
extern Unsigned1 nrPieces[2];
extern Unsigned1 pieceIndex[H8 + 1];
extern BoardIndexType kingSquare[2];
extern char     gameString[MaxGamePlies][MaxMoveLength];
extern MoveType game[MaxGamePlies];
extern Unsigned2 nrGamePlies,
                maxGamePlies;
extern Unsigned2 startPly;
extern SquareType computer;
extern Unsigned4 nrInternalNodes,
                nrLeafNodes,
                nrQuiesNodes;
extern Unsigned4 nrIllegalMoves;
extern Unsigned4 nrTotalNodes,
                nrExtraNodes;
extern double   startingTime;
extern Unsigned2 nrAllocMoveLists,
                nrAllocPathMoves;
extern Boolean  stopSearching;
extern char    *moveColorString[2];
#ifdef Bcc
extern huge Unsigned2 historyTable[2][H8 + 1][H8 + 1];
#else
extern Unsigned2 historyTable[2][H8 + 1][H8 + 1];
#endif
extern Signed2  pawnEvalTable[2][H8 + 1];
extern Signed2  knightEvalTable[2][H8 + 1];
extern Signed2  bishopEvalTable[2][H8 + 1];
extern Signed2  rookEvalTable[2][H8 + 1];
extern Signed2  queenEvalTable[2][H8 + 1];
extern Signed2  kingEvalTable[2][H8 + 1];
extern Signed2  centreTable[H8 + 1];
extern Boolean  interActive,
                doTest,
                useTransTable,
                useRefTable;
extern Boolean  useTwoLevelTable,
                useTtValue,
                useTtMove,
                timeStamp;
extern Boolean  useFlag;
extern char    *version;
extern TtEntry *transTable;
extern Unsigned4 nrCollisions,
                nrHits,
                nrReads,
                nrWrites;
extern Signed4  testNrNodes;
//extern Unsigned1 nrTtBits;
extern Unsigned2 keepPly;
extern Unsigned1 keepMethod;
extern char    *methodString[5];
extern Unsigned2 debugDepth;

extern char    *logicString[2];

extern Unsigned1 packPiece[Edge];
extern SquareType unpackPiece[4];       /* 2 bits */
extern Unsigned1 packSquare[H8 + 1];
extern BoardIndexType unpackSquare[64]; /* 6 bits */
extern Unsigned1 packType[64];
extern Unsigned1 unpackType[8]; /* 3 bits */

#ifdef Bcc
extern huge BoardIndexType knightTable[H8 + 1][MaxKnightMoves];
extern huge BoardIndexType bishopTable[H8 + 1][4][MaxBishopMoves];
extern huge BoardIndexType rookTable[H8 + 1][4][MaxRookMoves];
extern huge BoardIndexType pawnMoveTable[2][H8 + 1][2];
extern huge BoardIndexType pawnCaptureTable[2][H8 + 1][2];
#else
extern BoardIndexType knightTable[H8 + 1][MaxKnightMoves];
extern BoardIndexType bishopTable[H8 + 1][4][MaxBishopMoves];
extern BoardIndexType rookTable[H8 + 1][4][MaxRookMoves];
extern BoardIndexType pawnMoveTable[2][H8 + 1][2];
extern BoardIndexType pawnCaptureTable[2][H8 + 1][2];
#endif

extern DirectionType knightDir[8],
                bishopDir[4],
                rookDir[4];
extern DirectionType pawnCaptureDir[2][2];

extern char    *commandString[MaxCommands][2];

extern HashType pawnHashTable[2][H8 + 1];
extern HashType knightHashTable[2][H8 + 1];
extern HashType bishopHashTable[2][H8 + 1];
extern HashType rookHashTable[2][H8 + 1];
extern HashType queenHashTable[2][H8 + 1];
extern HashType kingHashTable[2][H8 + 1];
extern HashType movedHash;
extern HashType enPassantHashTable[H + 1];
extern HashType castlingHashTable[2][2];

extern Boolean  printDebug;
extern Unsigned4 nrMovesGenerated,
                nrGenerates;
